use crate::components::header::Header;
use crate::models::HoldPosition;
use crate::models::HoldRole;
use crate::models::Wall;
use leptos::prelude::*;
use serde::Deserialize;
use serde::Serialize;

#[component]
pub fn EditWall() -> impl leptos::IntoView {
    let load = async move {
        // TODO: What to do about this unwrap?
        load_initial_data().await.unwrap()
    };

    leptos::view! {
        <div class="min-w-screen min-h-screen bg-slate-900">
        <div class="container mx-auto">
            <Header />
            <Await future=load let:data>
                <Ready data=data.to_owned() />
            </Await>
        </div>
        </div>
    }
}

#[component]
fn Ready(data: InitialData) -> impl leptos::IntoView {
    leptos::logging::log!("ready");
    let mut hold_positions = vec![];
    for row in 0..(data.wall.rows) {
        for col in 0..(data.wall.cols) {
            hold_positions.push(HoldPosition { row, col });
        }
    }

    let mut cells = vec![];
    for &_hold_position in &hold_positions {
        let cell = view! { <Hold /> };
        cells.push(cell);
    }

    let grid_classes = format!("grid grid-rows-{} grid-cols-{} gap-4", data.wall.rows, data.wall.cols);

    view! {
        <div class=move || { grid_classes.clone() }>{cells}</div>
    }
}

#[component]
fn Hold() -> impl leptos::IntoView {
    view! { <div class="bg-indigo-100 aspect-square rounded"></div> }
}

#[derive(Serialize, Deserialize, Clone)]
pub struct InitialData {
    wall: Wall,
}

#[server]
async fn load_initial_data() -> Result<InitialData, ServerFnError> {
    use crate::server::state::State;

    let state = expect_context::<State>();

    let wall = state.persistent.with(|s| s.wall.clone()).await;
    Ok(InitialData { wall })
}
